from __future__ import annotations
import ast
from typing import Tuple, TypeVar, Iterable
from func_adl import ObjectStream, func_adl_callback, func_adl_parameterized_call
from enum import Enum
import func_adl_servicex_xaodr22

_method_map = {
    'nRingSets': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'nRingSets',
        'return_type': 'unsigned int',
    },
    'size': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'size',
        'return_type': 'unsigned int',
    },
    'at': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'at',
        'return_type': 'const xAOD::RingSet_v1 *',
    },
    'ringSetLinks': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'ringSetLinks',
        'return_type_element': 'ElementLink<DataVector<xAOD::RingSet_v1>>',
        'return_type_collection': 'const vector<ElementLink<DataVector<xAOD::RingSet_v1>>>',
    },
    'ringAt': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'ringAt',
        'return_type': 'float',
    },
    'index': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'index',
        'return_type': 'int',
    },
    'usingPrivateStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'usingPrivateStore',
        'return_type': 'bool',
    },
    'usingStandaloneStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'usingStandaloneStore',
        'return_type': 'bool',
    },
    'hasStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'hasStore',
        'return_type': 'bool',
    },
    'hasNonConstStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'hasNonConstStore',
        'return_type': 'bool',
    },
    'clearDecorations': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'clearDecorations',
        'return_type': 'bool',
    },
    'trackIndices': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'trackIndices',
        'return_type': 'bool',
    },
    'auxdataConst': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'auxdataConst',
        'return_type': 'U',
    },
    'isAvailable': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::CaloRings_v1',
        'method_name': 'isAvailable',
        'return_type': 'bool',
    },
}

_enum_map = {      
}

T = TypeVar('T')


def _add_method_metadata(s: ObjectStream[T], a: ast.Call) -> Tuple[ObjectStream[T], ast.Call]:
    '''Add metadata for a collection to the func_adl stream if we know about it
    '''
    assert isinstance(a.func, ast.Attribute)
    if a.func.attr in _method_map:
        s_update = s.MetaData(_method_map[a.func.attr])

        s_update = s_update.MetaData({
            'metadata_type': 'inject_code',
            'name': 'xAODCaloRings/versions/CaloRings_v1.h',
            'body_includes': ["xAODCaloRings/versions/CaloRings_v1.h"],
        })


        s_update = s_update.MetaData({
            'metadata_type': 'inject_code',
            'name': 'xAODCaloRings',
            'link_libraries': ["xAODCaloRings"],
        })

        for md in _enum_map.get(a.func.attr, []):
            s_update = s_update.MetaData(md)
        return s_update, a
    else:
        return s, a


@func_adl_callback(_add_method_metadata)
class CaloRings_v1:
    "A class"


    def nRingSets(self) -> int:
        "A method"
        ...

    def size(self) -> int:
        "A method"
        ...

    def at(self, index: int) -> func_adl_servicex_xaodr22.xAOD.ringset_v1.RingSet_v1:
        "A method"
        ...

    def ringSetLinks(self) -> func_adl_servicex_xaodr22.vector_elementlink_datavector_xaod_ringset_v1___.vector_ElementLink_DataVector_xAOD_RingSet_v1___:
        "A method"
        ...

    def ringAt(self, rsIdx: int, ringIdx: int) -> float:
        "A method"
        ...

    def index(self) -> int:
        "A method"
        ...

    def usingPrivateStore(self) -> bool:
        "A method"
        ...

    def usingStandaloneStore(self) -> bool:
        "A method"
        ...

    def hasStore(self) -> bool:
        "A method"
        ...

    def hasNonConstStore(self) -> bool:
        "A method"
        ...

    def clearDecorations(self) -> bool:
        "A method"
        ...

    def trackIndices(self) -> bool:
        "A method"
        ...

    @func_adl_parameterized_call(lambda s, a, param_1: func_adl_servicex_xaodr22.type_support.cpp_generic_1arg_callback('auxdataConst', s, a, param_1))
    @property
    def auxdataConst(self) -> func_adl_servicex_xaodr22.type_support.index_type_forwarder[func_adl_servicex_xaodr22.str.str]:
        "A method"
        ...

    @func_adl_parameterized_call(lambda s, a, param_1: func_adl_servicex_xaodr22.type_support.cpp_generic_1arg_callback('isAvailable', s, a, param_1))
    @property
    def isAvailable(self) -> func_adl_servicex_xaodr22.type_support.index_type_forwarder[func_adl_servicex_xaodr22.str.str]:
        "A method"
        ...
